import os.path
import string
# Prop[erties from here
# http://msdn.microsoft.com/library/default.asp?url=/library/en-us/vcext/html/vcoriVisualCExtensibilitySharedAllProperties.asp
#
# Tool rule setting
# Sometimes file setting s override trhe config rule and sometimes they
# just add to it maybe this is wrong?
# Copyright 2003 Gareth Bushell

TOOL_RULE_OVERRIDE = 0
TOOL_RULE_BOTH = 1

currentconfig = None
rules_module = None

MAKEFILE_CONFIG_TYPE = u'0'
DLL_CONFIG_TYPE = u'1'
EXE_CONFIG_TYPE = u'2'

# Should move these rules into a differnt file
# msvc7_rules?

makefile_toolextmap = { '.c' : None }

exe_dll_toolextmap = {'.c' : 'VCCLCompilerTool',
                      '.rc': 'VCResourceCompilerTool',
                      '.dll' : 'VCLinkerTool',
                      '.exe' : 'VCLinkerTool',
                      '.obj' : None,
                      '.ico' : None,}

toolextmap = { MAKEFILE_CONFIG_TYPE : makefile_toolextmap,
               DLL_CONFIG_TYPE : exe_dll_toolextmap,
               EXE_CONFIG_TYPE : exe_dll_toolextmap }

linktoolname_map = { MAKEFILE_CONFIG_TYPE :'VCNMakeTool',
                     DLL_CONFIG_TYPE : 'VCLinkerTool',
                     EXE_CONFIG_TYPE : 'VCLinkerTool' }


source2objmap = {'.obj' : 'VCCLCompilerTool',
                 '.res': 'VCResourceCompilerTool'}

# HMM are the ones I'm not sure about...
OPTION_TYPE_HMM = 0
OPTION_TYPE_BOOL = 1 
OPTION_TYPE_BOOL_PARAM = 2
OPTION_TYPE_MAP = 3
OPTION_TYPE_PARAM = 4 
OPTION_TYPE_LIST = 5

# IGNORE is there as they are probably used in some other
# parameter thing or are just infomation (Name)
OPTION_TYPE_IGNORE = 6

def set_rules( rule_module ):
    """
    Sets the rule set to translate the Visual studio
    Options to the compiler options

    Parameters:

    * rule_module - The rule module to use

    Returns

    * None
    """
    global rules
    global rules_module
    rules = rule_module.rules
    rules_module =  rule_module

def set_config( config ):
    """
    Sets the current Project configuration to
    use

    Parameters:

    * config - The config section of the [project dictionary loaded in vcproj_Read.load_project

    Returns:

    * None
    """
    global currentconfig
    currentconfig = config

def find_file_config( fileconfig ):
    for config in fileconfig['FileConfiguration']:
        if config['Name'] == currentconfig['Name']:
            return config

def find_tool_settings( fileconfig, name, setting, rule):
    """
    Gets the current named file setting. It looks up the
    tool (name) and retrives the named setting (setting).
    If this setting it uses the global setting according the
    how rule is set

    Parameters:

    * fileconfig - A config structure for a file

    * name (string) - The name of the tool to retrive the setting for

    * setting (string) - The name of the setting

    * rule (enum) - The following action taken on the values:

      * TOOL_RULE_BOTH - Combines the setting in the file and current config

      * TOOL_RULE_OVERRIDE - Uses the first setting it retrives

    Returns:

    * The current value of the setting dependend on rule

      * TOOL_RULE_BOTH - Returns a tuple of length 2. The first value is from the fileconfig

      * TOOL_RULE_OVERRIDE - Return the setting from fileconfig or global config if not avil in fileconfig.
      
    Exceptions:

    * KeyError - Raised if the setting or tool isn't found
    """
    settings = []
    try:
        currentfileconfig = find_file_config( fileconfig )
        for tool in currentfileconfig['Tool']:
            if tool['Name'] == name:
                if tool[setting] != '':
                    settings.append( tool[setting] )
                if rule == TOOL_RULE_OVERRIDE:
                    return tool[setting]
    except KeyError:
        # OK no file specific config for this file
        pass
    
    for tool in currentconfig['Tool']:
        if tool['Name'] == name:
            settings.append( tool[setting] )
            if rule == TOOL_RULE_BOTH:
                return settings
            else:
                return tool[setting]
            
    raise KeyError,"DAMN" + str(fileconfig) + str(name) + str(setting)

def option_list( fconfig, tool, var, optioninfo ):
    """
    """
    options = find_tool_settings( fconfig, tool, var, TOOL_RULE_BOTH)
    optionlist = []
    for option in options:
        if len(optioninfo['delim']) > 1:
            option = string.replace( option,
                                     optioninfo['delim'][1:],
                                     optioninfo['delim'][0] * ( len(optioninfo['delim']) - 1) )
            
        optionlist += option.split( optioninfo['delim'][0] )

    return optioninfo['flag'] + optioninfo['flag'].join( optionlist )

def option_param( fconfig, tool, var, optioninfo ):
    """
    """
    option = find_tool_settings( fconfig, tool,var,TOOL_RULE_OVERRIDE)
    return optioninfo['flag'] + option

def option_bool( fconfig, tool, boolname, optioninfo ):
    """
    """
    line = ""
    b = find_tool_settings( fconfig, tool, boolname, TOOL_RULE_OVERRIDE)
    if bool(b):
        line = optioninfo['flag']
    return line
        
def option_bool_param(fconfig, tool, boolname, optioninfo ):
    """oinfo['paramname'], oinfo['flag']
    """
    line = ""
    b = find_tool_settings( fconfig,tool,boolname,TOOL_RULE_OVERRIDE)
    if bool(b):
        paramvalue = find_tool_settings( fconfig,
                                         tool,
                                         optioninfo['paramname'],
                                         TOOL_RULE_OVERRIDE )
        line = optioninfo['flag'] + paramvalue
    return line


def option_map( fconfig, tool, var, optioninfo ):
    """
    """
    option = find_tool_settings( fconfig, tool, var,TOOL_RULE_OVERRIDE)
    return optioninfo['map'][ option ]

def option_hmm( fconfig, tool, var, optioninfo ):
    """
    """
    print "Option ", option, "Not implemented yet!"
    return ""

def option_ignore( fconfig, tool, var, optioninfo ):
    return ""

option_function_map = { OPTION_TYPE_HMM : option_hmm,
                        OPTION_TYPE_BOOL : option_bool,
                        OPTION_TYPE_BOOL_PARAM : option_bool_param,
                        OPTION_TYPE_MAP : option_map,
                        OPTION_TYPE_PARAM : option_param,
                        OPTION_TYPE_LIST : option_list,
                        OPTION_TYPE_IGNORE : option_ignore }

def run_option( f, tool, option ):
    """
    """
    line = ""
    oinfo = rules[tool][option]

    return option_function_map[ oinfo['type'] ]( f, tool, option, oinfo )
        

## def generate_depends_line( f ):
##     """
##     """
##     pass

## def generate_make_line( f ):
##     """
##     """
##     find_tool_settings( f, , rules[ toolname ]['objpath'] ,TOOL_RULE_OVERRIDE)

def generate_compile_line( f ):
    """
    Generate a line for the specified file
    config. The global config must be set
    """
    cmd = {}
    # Get the extention of the file and find
    # The tool to compile it.
    rest, ext = os.path.splitext( str(f['RelativePath']) )
    #print rest,ext

    
    toolname = toolextmap[currentconfig['ConfigurationType']][ ext ]
    if toolname:
        lines = []
        for tool in currentconfig['Tool']:
            if tool['Name'] == toolname:
                for setting in tool:
                    lines.append( run_option( f, toolname, setting) )

        cmd['pre'] = rules[ toolname ]['pre']

        cmd['flags'] =  " ".join( lines  )
        
        cmd['post'] =  rules[ toolname ]['post']
        cmd['source'] = f['RelativePath']
        
        if rules[ toolname ]['objpath']:
           objectpath = find_tool_settings( f, toolname, rules[ toolname ]['objpath'] ,TOOL_RULE_OVERRIDE)
        else:
            objectpath = os.path.split( f['RelativePath'] )[0]
            
        objectfile = os.path.splitext( os.path.split( f['RelativePath'] )[1] )[0] + "." + rules[ toolname ]['objext'] 

        cmd['out'] = os.path.join(objectpath,objectfile)
        return ( cmd )
    return None


def generate_link_line( objs  ):
    """
    """
    f = {}
    # Select a tool
    toolname = linktoolname_map[currentconfig['ConfigurationType']]
    
    lines = []
    for tool in currentconfig['Tool']:
        if tool['Name'] == toolname:
            for setting in tool:
                o = run_option( f, toolname, setting)
                lines.append( o )
                #print o
    

    lines.append( rules_module.ConfigurationTypemap[ currentconfig['ConfigurationType'] ] )

    lines += objs

    cmd = {}
    cmd['pre'] = rules[ toolname ]['pre']
    cmd['post'] = rules[ toolname ]['post']
    cmd['flags'] =  " ".join( lines )
    cmd['out'] = find_tool_settings( f, toolname,  rules[ toolname ]['outputkey'], TOOL_RULE_OVERRIDE)
        
    
    #print lines
    return cmd



## def generate_target_line( sources, target  ):
##     """
##     """
##     f = {}

##     lines  = generate_compile_line( target )

##     lines += objs
##     lines.append( rules_module.ConfigurationTypemap[ currentconfig['ConfigurationType'] ] )
##     return " ".join( lines )

## def generate_command_line_old( f ):
##     """
##     """
##     ext = f['RelativePath'].split(".")[-1]
##     cmdlinefunc = filetype[ ext ]
##     return cmdlinefunc( f )


def generate_custom_build_step( ):
    """
    """
    return run_option( {}, 'VCCustomBuildTool', 'CommandLine') 

                

## def generate_make_rule( sources, target ):
##     """
##     create a dictionary that describe how to
##     create target from sources
##     """

##     #print "sources",sources
##     #print "target",target
##     buildcmds = []
##     for source in sources:
##         #print "Source=",source
##         line = generate_compile_line( source )
##         if line:
##             buildcmds.append( line )

##     #print "buildcmds",buildcmds
##     raise ""

##     objs = []
##     for cmd in buildcmds:
##         objs.append( cmd['out'] )
    
##     line = generate_link_line( objs )
##     if line:
##         buildcmds.append( line )

##     return { target : { 'deps' : '',
##                         'rules' : buildcmds,
##                         } }
