# Copyright 2003 Gareth Bushell
import os
import sys
from xml.dom.ext.reader import Sax2
import xml.dom.ext
import file_rules
import gnu_make

def attributes2dict(dom):
    """
    """
    d = {}
    for attr in dom.attributes:
        #print attr
        d[attr.name] = attr.nodeValue
    return d

def xmlnode2dict( dom ):
    """
    Converts an XML document into a dictionary
    maybe not in a correct manner but it'll do for vcproj files
    
    """
    self = attributes2dict( dom )
    if dom.hasChildNodes():
        for child in dom.childNodes:
            try:
                childdict = xmlnode2dict( child )
                try:
                    self[child.nodeName].append( childdict )
                except KeyError:
                   self[child.nodeName] = [ childdict ]
            except TypeError:
                pass #print "skipping",child
    return self


def load_project( filename ):
    xml_file = file( filename, 'rU' )

    reader = Sax2.Reader()

    doc = reader.fromStream( xml_file )
    
    xml_file.close()

    vsproj = doc.getElementsByTagName('VisualStudioProject')[0]

    project = xmlnode2dict( vsproj )

    return project



def get_config( project, configname ):
    #import pprint
    #pprint.pprint( project )
    #print configname
    for config in project['Configurations'][0]['Configuration']:
        if config['Name'] == configname:
            currentconfig = config
    try:
        print "Config selected",  currentconfig['Name']
    except NameError:
        print "Couldn't find config",sys.argv[2]
        sys.exit(1)
        
    return currentconfig


def writecmdline( cmdline ):
    clf = file( 'link.txt','w')
    clf.write( cmdline )
    clf.close

# write the makefile  by sending ocommand off to MakfileWrite class
# For now just exe

def write_makfile_file( target, deps, line ):
    pass


def exe_dll_build( project, currentconfig ):
    # make temp build directory
    try:
        print "Trying to make ",currentconfig['IntermediateDirectory'] 
        os.makedirs( currentconfig['IntermediateDirectory'] )
    except OSError:
        pass

    targets = {}
    objs = []
    sources = []
    for f in project['Files'][0]['File']:
        o = file_rules.generate_compile_line( f )
        if o:
            targets.update( { o['out'] :{  'deps' : [o['source']], # I thinks source should go here
                                           'rules' : ["%(pre)s %(flags)s %(post)s %(source)s" % o] }})
            # make a list of all the objects
            objs.append( o['out'] )

            # And the source (Not really sure if this may be usefull for dependencies
            sources.append( o['source'] )
            
    
    makedef = {}
    
    makedef['targets'] = targets
    
    linkline =  file_rules.generate_link_line( objs )

    linktarget = { linkline['out'] :{  'deps' : objs ,
                                       'rules' : ["%(pre)s %(flags)s %(post)s" % linkline] }}


    # Any custom build steps
    try:
        customline = file_rules.generate_custom_build_step()
        #print customline
        linktarget[linkline['out']]['rules'].append( customline )
    except:
        print "No custom build step(I Hope!)"
        pass

    #print linkline
    makedef['targets'].update( linktarget )


    
    makedef['all'] = linkline['out']
    makedef['phoney'] = []

    #import pprint
    #pprint.pprint( makedef )

    rest, ext = os.path.splitext( linkline['out'] )

    makefilename = gnu_make.write_make( rest,  makedef )
    
    return makefilename

    
def vcproj2make( projectname, configname ):
    project = load_project( projectname )
    currentconfig = get_config( project, configname ) 
    file_rules.set_config( currentconfig )

    #import pprint
    #pprint.pprint( currentconfig )

    # Work out what to do..
    # http://msdn.microsoft.com/library/default.asp?url=/library/en-us/vcext/html/vxlrfvcprojectenginelibraryconfigurationtypes.asp
    
    if currentconfig[u'ConfigurationType'] in [u'0',u'1',u'2']:
        # Makefile, DLL, exe Someother type of project
        return exe_dll_build( project, currentconfig )
    else:
        print "Bugger unsupported configuration type"
        sys.exit(1)
        
    
    

def compile_it( project ):
    """
    """
    #filedb = {}
    objectlist = []
    import os
    for f in project['Files'][0]['File']:
        print "Processing ",f['RelativePath']
    
        cmddict =  file_rules.generate_compile_line( f )
        if cmddict:
            #print cmddict
            cmdline = '%(pre)s %(flags)s %(post)s %(source)s' % cmddict

            obj = cmddict['out']
            objectlist.append(  obj )
            if cmdline:
                os.system( cmdline )

    #print "Now link.....    #print "link ",objectlist
    # print "When I get around to writing it"

    linkline =  file_rules.generate_link_line( objectlist )

    writecmdline( linkline )

    #print linkline

    os.system( "link @link.txt" )

    try:
        customline = file_rules.generate_custom_build_step()
        #print customline
        os.system( customline )
    except:
        print "No custom build step(I Hope!"
        pass


if __name__ == '__main__':
    if len(sys.argv) < 3:
        print "<XML_PROJECT FILE> <CONFIG> [rules]"
        sys.exit(1)

    # Specify the rules set
    if len(sys.argv) == 4:
        rules = __import__( sys.argv[3] )
        rules = file_rules.set_rules( rules )
    else:
        import msvc7_rules
        file_rules.set_rules( msvc7_rules )

    print "Makeing ",vcproj2make( sys.argv[1], sys.argv[2] )
